#ifndef MPC_PASSAGEWAYTREE_HPP
#define MPC_PASSAGEWAYTREE_HPP

#include "avoidObs/inc/Passageway.hpp"
// #include "avoidObs/inc/GraphPassageway.hpp"
#include "common/inc/Tree.hpp"
#include "system/inc/System.hpp"
#include <vector>

namespace mpc
{
//   typedef struct WAYPOINVT
//   {
//     XYThS_Config cfg;               /**< configuration (x,y,th) of the waypoint */
//     double v;                       /**< linear velocity */
//     double t;                       /**< time */
//   } WayPointVT;


                                    /**************************************************/
                                    /*************** Class PassagewayNode *************/
                                    /**************************************************/

  /** \class PassagewayNode
   *  \brief directed graph class with passageway vertex values
   */
  class PassagewayNode : public Node<PassagewayWP*>
  {
    /********** Constructors and destructors **********/

  protected : 
    /**
     *  \brief default constructor
     */
    PassagewayNode();

  public : 
    /**
     *  \brief copy constructor
     */
    PassagewayNode(const PassagewayNode& n);

    /**
     *  \brief init constructor
     */
    PassagewayNode(const Passageway& p, const StateTime& statet);

//     /**
//      *  \brief init constructor (from a passageway graph and system constraints)
//      *  \param pgraph : the input passageway graph
//      *  \param sys : the input system constraints
//      */
//     PassagewayNode(const GraphPassageway& pgraph, const System& sys);

    /**
     *  \brief destructor
     */
    virtual ~PassagewayNode();

    /**************** Utility function ****************/

  private : 
    /**
     *  \brief return a vector containing the state-times (for a differential drive system) that constitute the best branch
     *  \return a vector containing the state-times (for a differential drive system) that constitute the best branch
     */
    std::vector<PassagewayWP>* bestBranch_PassagewayWP();
    
  public : 
    /**
     *  \brief return a vector containing the state-times (for a differential drive system) that constitute the best branch
     *  \return a vector containing the state-times (for a differential drive system) that constitute the best branch
     */
    std::vector<DD2StateT>* bestBranch_DD2StateT();

//     /************ Passageway tree building ************/

//   protected : 
//     /*
//      *  \brief build a passageway tree from a a passageway graph and system constraints
//      *  \param pgraph : the input passageway graph
//      *  \param sys : the input system constraints
//      */
//     virtual void buildFromPGraph(const GraphPassageway& pgraph, const System& sys);
  };
};
  
#endif // MPC_PASSAGEWAYTREE_HPP
