#ifndef CONTROLPOINT_HPP
#define CONTROLPOINT_HPP

#include "common/inc/Position.hpp"

namespace mpc
{
                                    /**************************************************/
                                    /*************** Class ControlPoint ***************/
                                    /**************************************************/

  /** \class ControlPoint2D
   * \brief control point over a system 
   *
   * Class defining the control points defined
   * over the system considered in the workspace (W = R^2)
   */
  class ControlPoint2D : public PositionXY
  {
  protected : 
    //   double x_;                        /**< X coordinate of the control point */
    //   double y_;                        /**< Y coordinate of the control point */

  public : 
    /********** Constructors and destructors **********/

    /**
     *  \brief default constructor
     */
    ControlPoint2D();
    /**
     *  \brief copy constructor
     *  \param c : the control point to copy
     */
    ControlPoint2D(const ControlPoint2D& c);
    /**
     *  \brief init constructor
     *  \param x : x coord of the control point
     *  \param y : y coord of the control point
     */
    ControlPoint2D(double x,double y);
    /**
     *  \brief destructor
     */
    virtual ~ControlPoint2D();
  
    //   /************** Getters and setters ***************/

    //   /**
    //    *  \brief X coord accessor
    //    *  \return reference to the X coord
    //    */
    //   inline double x() const { return this->x_; }
    //   /**
    //    *  \brief Y coord accessor
    //    *  \return reference to the Y coord
    //    */
    //   inline double y() const { return this->y_; }
  };

};
#endif // CONTROLPOINT_HPP


#ifndef CONTROLPOINT3D_HPP
#define CONTROLPOINT3D_HPP

namespace mpc
{
                                    /**************************************************/
                                    /*************** Class ControlPoint ***************/
                                    /**************************************************/

  /** \class ControlPoint3D
   * \brief control point over a system 
   *
   * Class defining the control points defined
   * over the system considered in the workspace (W = R^3)
   */
  class ControlPoint3D : public PositionXYZ, public ControlPoint2D
  {
  protected : 
    //   double z_;                        /**< Z coordinate of the control point */

  public : 
    /********** Constructors and destructors **********/

    /**
     *  \brief default constructor
     */
    ControlPoint3D();
    /**
     *  \brief copy constructor
     *  \param c : the control point to copy
     */
    ControlPoint3D(const ControlPoint3D& c);
    /**
     *  \brief init constructor
     *  \param x : x coord of the control point
     *  \param y : y coord of the control point
     */
    ControlPoint3D(double x,double y,double z);
    /**
     *  \brief destructor
     */
    virtual ~ControlPoint3D();
  
    //   /************** Getters and setters ***************/

    //   /**
    //    *  \brief Y coord accessor
    //    *  \return reference to the Y coord
    //    */
    //   inline double z() const { return this->z_; }
  };

};

#endif // CONTROLPOINT3D_HPP
