#ifndef MPC_POSITIONXYT_HPP
#define MPC_POSITIONXYT_HPP

#include "common/inc/Position.hpp"

namespace mpc
{
                                    /**************************************************/
                                    /**************** Class PositionXYT ***************/
                                    /**************************************************/

  /** \class PositionXYT
   *  \brief 2D Position in the workspace increased by the time dimension (XxYxT)
   */
  class PositionXYT : public PositionXY
  {
  protected : 

  public : 
    /********** Constructors and destructors **********/

    /**
     *  \brief default constructor
     */
    PositionXYT();
    /**
     *  \brief copy constructor
     *  \param p : the position to copy
     */
    PositionXYT(const PositionXYT& p);
    /**
     *  \brief init constructor
     *  \param x : the first position coord
     *  \param y : the second position coord
     */
    PositionXYT(double x,double y,double t);
    /**
     *  \brief destructor
     */
    virtual ~PositionXYT();

    /******************** Accessors *******************/

    /**
     *  \brief accessor of the first coordinate
     *  \return the first coordinate of the position
     */
    inline double& t() { return (*this)[2]; }

    /**
     *  \brief accessor of the first coordinate
     *  \return the first coordinate of the position
     */
    inline double t() const { return this->at(2); }

    /******************* Operators ********************/

    /**
     *  \brief operator = : assignation
     *  \param p2 : the position-time to assign to the current one
     */
    void operator=(const PositionXYT& p2);

    /**
     *  \brief operator + : sumation
     *  \param p2 : the second vector to add
     *  \return the sum of both positions
     */
    PositionXYT operator+(const PositionXYT& p2) const;
  
    /**
     *  \brief operator + : substraction
     *  \param p2 : the second position to substract
     *  \return the substraction of both positions
     */
    PositionXYT operator-(const PositionXYT& p2) const;

    /**
     *  \brief operator * : scalar multiplication
     *  \param d : the scalar to multiply with
     *  \return the product of the current position and the scalar
     */
    PositionXYT operator*(double d) const;

    /**
     *  \brief operator / : division by a scalar
     *  \param d : the scalar to divide with
     *  \return the division result
     */
    PositionXYT operator/(double d) const;

    /**
     *  \brief operator ^ : cross product
     *  \param p2 : the second vector
     *  \return the cross product of both vectors
     */
    PositionXYT operator^(const PositionXYT& p2) const;

  };

};
#endif // MPC_POSITIONXYT_HPP

#ifndef MPC_POSITIONXYThT_HPP
#define MPC_POSITIONXYThT_HPP

namespace mpc
{
                                    /**************************************************/
                                    /*************** Class PositionXYYhT **************/
                                    /**************************************************/

  /** \class PositionXYThT
   *  \brief 3D Position in the workspace increased by the time dimension (XxYxThxT)
   */
  class PositionXYThT : public PositionXYTh
  {
  protected : 

  public : 
    /********** Constructors and destructors **********/

    /**
     *  \brief default constructor
     */
    PositionXYThT();
    /**
     *  \brief copy constructor
     *  \param p : the position to copy
     */
    PositionXYThT(const PositionXYThT& p);
    /**
     *  \brief init constructor
     *  \param x : the first position coord
     *  \param y : the second position coord
     */
    PositionXYThT(double x,double y,double th,double t);
    /**
     *  \brief destructor
     */
    virtual ~PositionXYThT();

    /******************** Accessors *******************/

    /**
     *  \brief accessor of the first coordinate
     *  \return the first coordinate of the position
     */
    inline double& t() { return (*this)[3]; }

    /**
     *  \brief accessor of the first coordinate
     *  \return the first coordinate of the position
     */
    inline double t() const { return this->at(3); }

    /******************* Operators ********************/

    /**
     *  \brief operator = : assignation
     *  \param p2 : the position-time to assign to the current one
     */
    void operator=(const PositionXYThT& p2);
  };

};
#endif // MPC_POSITIONXYT_HPP
