#ifndef _POSITION_HPP_
#define _POSITION_HPP_

#include "environment/inc/Map.hpp"
//#include "Point.hpp"

#include <iostream>
#include <vector>

using namespace std;

#ifndef _DIR_T_
#define _DIR_T_

	// Various directions
	typedef enum {
		TMP  = -2, // temporary direction 
		IMP  = -1, // impossible direction
		UND  =  0, // undefined direction
		NW   =  1, // north west
		N    =  2, // north
		NE   =  3, // north east
		E    =  4, // east
		SE   =  5, // south east
		S    =  6, // south
		SW   =  7, // south west
		W    =  8  // west
	} dir_t;

#endif

namespace mpc
{
class Map;

class Pos{

	private : 
  		int x;
  		int y;

	public : 
  		/********************** constructors and destructors **********************/
  		Pos();                                        // default constructor
  		Pos(int x,int y);                             // init constructor (from coord (x,y))
	        Pos(const Pos &p,dir_t d);                    // init constructor (from position p and direction d)
  //  		Pos(Point2D p);                               // init constructor (from Point2D)
  		Pos(const Pos &p);                            // copy constructor
  		~Pos();                                       // destructor

                /********************************* getters ********************************/
  		int getX() const;                                  // return position's absicca
  		int getY() const;                                  // return position's ordinate
  		bool equals(const Pos &p);                    // return true if both positions get same cordinates
 
                /***************************** reachability *******************************/
  		bool isAvailable(Map *map);                        
                  // return true if this position is available on the map
  		bool isReachable(Map *map);                        
                  // return true if this position is reachable on this map (there is no obstacle on this position)
  		vector<Pos> reachablePos(Map *map);           
                  // return a vector of all positions which are reachable by the current position
  		vector<dir_t> reachableDir(Map *map);              
                  // return a vector of all reachable direction from the current position
  		vector<dir_t> reachableDir1(Map *map);             
  		  // return a vector of all reachable direction from the current position (consider only N,E,S,W positions)
  		vector<dir_t> reachableDir2(Map *map);             
  		  // return a vector of all reachable direction from the current position (consider NW,N,NE,E,SE,S,SW,W positions)
  		vector<dir_t> availableDir(Map *map);              
                  // return a vector of all available direction from the current position (consider only N,E,S,W positions)
  
                /****************************** visibility ********************************/
  		vector<Pos>* getLine(Pos p);             
                  // get a line path between the current position and a target position
  		bool isVisible(Pos p,Map* map);               
  		  // return true if position p is visible by current position

                /******************************* distances ********************************/
  		double getDist(Pos p);                // return the distance between this position and an other one
  		int searchNearObs(Map* map);               // search the distance to the nearest obstacle on the map

  		/********************** static method for directions **********************/
  		static dir_t oppDir(dir_t dir);                    // return opposite direction to direction dir
  		static dir_t privDir(Pos* p1,Pos* p2);   // return privileged direction from p1 to p2
  		static vector<dir_t>* srdDir(dir_t dir);           // return surrounded direction of dir

                /**************************** drawing functions ***************************/
  		void drawOn(Map* map,int col);                     // draw the point on the map
};

};
#endif
