#ifndef DIFFDRIVESYS_HPP
#define DIFFDRIVESYS_HPP

#include "system/inc/System.hpp"

namespace mpc
{
                                    /**************************************************/
                                    /*************** Class DiffDriveSys ***************/
                                    /**************************************************/

  /** \class DiffDriveSys
   * \brief Differential drive system class
   */
  class DiffDriveSys : public System
  {
  protected : 
    double length_;                                   /**< length of the robot (between the center and the wheels) */
    double wheelRad_;                                 /**< radius of the wheels of the robot */

    double radius_;                                   /**< radius of the cirle which include the robot */

    double vmax_;                                     /**< maximal speed */
    double amax_;                                     /**< maximal acceleration */
  
  public : 

    /********** Constructors and destructors **********/

    /**
     *  \brief default constructor
     */
    DiffDriveSys();

    /**
     *  \brief init constructor
     *  \param maxWVel : maximal wheel velocity
     *  \param maxWAcc : maximal wheel acceleration
     *  \param R : radius of the differential drive
     */
    DiffDriveSys(double maxWVel,double maxWAcc,double R);

    /**
     *  \brief destructor
     */
    virtual ~DiffDriveSys();

    /************** Getters and setters ***************/

    /**
     *  \brief getter : return the length of the robot (between the center and the wheels)
     *  \return the length of the robot (between the center and the wheels)
     */
    double getLength() const;
    /**
     *  \brief setter : set the length of the robot (between the center and the wheels)
     *  \param l : the new length of the robot (between the center and the wheels)
     */
    void setLength(const double& l);

    /**
     *  \brief getter : return the radius of the wheels
     *  \return the radius of the wheels
     */
    double getWheelRad() const;
    /**
     *  \brief setter : set the radius of the wheels
     *  \param l : the new radius of the wheels
     */
    void setWheelRad(const double& r);

    /**
     *  \brief getter : return the radius of the circle including the robot
     *  \return radius of the circle including the robot
     */
    double getRadius() const;
    /**
     *  \brief setter : set the radius of the circle including the robot
     *  \param r : radius of the circle including the robot
     */
    void setRadius(const double& r);
  
    /**
     *  \brief getter : return the maximal speed
     *  \return maximal speed
     */
    double getWVMax() const;
    /**
     *  \brief setter : set the maximal speed
     *  \param v : new maximal speed
     */
    void setWVMax(const double& v);
    /**
     *  \brief getter : return the maximal acceleration
     *  \return maximal acceleration
     */
    double getWAMax() const;
    /**
     *  \brief setter : set the maximal acceleration
     *  \param v : new maximal acceleration
     */
    void setWAMax(const double& a);

    /*************** Display functions ****************/

    /**
     *  \brief display the features of the differential drive system
     */
    virtual void display() const;

    /**
     *  \brief draw the differential drive system in an openGL viewer
     */
    virtual void toGL() const;
  };

};
#endif // DIFFDRIVESYS_HPP
